from datetime import datetime

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import smtplib

def GMAIL_SEND_MAIL(account_g, gmail_password, from_to, send_to, subject, message_html, slack_canal="default"):
	print(datetime.now(), "\tExecution ENVOIE MAIL")
	sender = account_g
	html = message_html
	msg = MIMEMultipart('alternative')
	msg['Subject'] = subject
	msg['From'] = from_to
	msg['To'] = "To Person <>"
	part = MIMEText(html, 'html')
	msg.attach(part)
	try:
		smtpObj = smtplib.SMTP('smtp-relay.gmail.com', port=587)
		smtpObj.ehlo()
		smtpObj.starttls()
		smtpObj.ehlo()		
		smtpObj.login('lpb_scripts@lepetitballon.com', gmail_password)
		smtpObj.sendmail(sender, send_to, msg.as_string())         
		smtpObj.quit()
		print(f"{datetime.now()}\tSuccessfully sent email to {send_to}")
#		sendMessageToSlack(slack_canal, "#0C7C59", f"Email envoyé à {send_to}\n avec l'objet:{subject}", f"{message_html}")
	except smtplib.SMTPException:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPException This exception is raised when the server unexpectedly disconnects, or when an attempt is made to use the SMTP instance before connecting it to a server.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPException This exception is raised when the server unexpectedly disconnects, or when an attempt is made to use the SMTP instance before connecting it to a server.")
	except smtplib.SMTPServerDisconnected:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPServerDisconnected This exception is raised when the server unexpectedly disconnects, or when an attempt is made to use the SMTP instance before connecting it to a server.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPServerDisconnected This exception is raised when the server unexpectedly disconnects, or when an attempt is made to use the SMTP instance before connecting it to a server.")
	except smtplib.SMTPResponseException:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("STMPResponseException Base class for all exceptions that include an SMTP error code. These exceptions are generated in some instances when the SMTP server returns an error code. The error code is stored in the smtp_code attribute of the error, and the smtp_error attribute is set to the error message.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "STMPResponseException Base class for all exceptions that include an SMTP error code. These exceptions are generated in some instances when the SMTP server returns an error code. The error code is stored in the smtp_code attribute of the error, and the smtp_error attribute is set to the error message.")
	except smtplib.SMTPSenderRefused:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPSenderRefused Sender address refused. In addition to the attributes set by on all SMTPResponseException exceptions, this sets ‘sender’ to the string that the SMTP server refused.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPSenderRefused Sender address refused. In addition to the attributes set by on all SMTPResponseException exceptions, this sets ‘sender’ to the string that the SMTP server refused.")
	except smtplib.SMTPRecipientsRefused:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPRecipientsRefused All recipient addresses refused. The errors for each recipient are accessible through the attribute recipients, which is a dictionary of exactly the same sort as SMTP.sendmail() returns.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPRecipientsRefused All recipient addresses refused. The errors for each recipient are accessible through the attribute recipients, which is a dictionary of exactly the same sort as SMTP.sendmail() returns.")
	except smtplib.SMTPDataError:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTP DataError The SMTP server refused to accept the message data.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTP DataError The SMTP server refused to accept the message data.")
	except smtplib.SMTPConnectError:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPConnectError Error occurred during establishment of a connection with the server.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPConnectError Error occurred during establishment of a connection with the server.")
	except smtplib.SMTPHeloError:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPHeloError The server refused our HELO message.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPHeloError The server refused our HELO message.")
	except smtplib.SMTPNotSupportedError:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPNotSupportedError The command or option attempted is not supported by the server.")
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPNotSupportedError The command or option attempted is not supported by the server.")
	except smtplib.SMTPAuthenticationError:
		print(f"{datetime.now()}\tError to sent email to {send_to}")
		print("SMTPAuthenticationError SMTP authentication went wrong. Most probably the server didn’t accept the username/password combination provided.")		
		#sendMessageToSlack(slack_canal, "#D64933", f"Erreur pour {send_to} \n avec l'objet:{subject}", "SMTPAuthenticationError SMTP authentication went wrong. Most probably the server didn’t accept the username/password combination provided.")


from datetime import datetime
import os 
import pandas as pd
from google.cloud import bigquery

def SQL_query(account_bigquery_secret, sql_path_folder, sql_path_file, last_execution_date = ""):
    """Execute a BigQuery SQL query and return the result as a DataFrame.
    :sql_path_file: The name of the SQL file to be executed.
    Returns:
        :df: A DataFrame containing the result of the SQL query.
    """
    sql_path = f"{sql_path_folder}/{sql_path_file}"
    with open(sql_path, "r") as file:
        sql = file.read()
    if last_execution_date != "":
        sql = sql.replace("WHERE date(updated_at) > DATE_SUB(CURRENT_DATE, INTERVAL 1 DAY)", f"WHERE date(updated_at) > DATE_SUB(\"{last_execution_date}\", INTERVAL 1 DAY)")
    
    os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = account_bigquery_secret
    client = bigquery.Client()
    df = client.query(sql).to_dataframe()
    return df



from datetime import datetime

class hour:
    now = datetime.now()
    current_time = now.strftime("%Y-%M-%d %H:%M:%S")


def computelog(header, msg, footer):
    print("="*50)
    print(f"{header}")
    print("="*50)
    print(f"{hour.current_time} Début de l'execution de la fonction")
    print(f"{msg}")
    print(f"{hour.current_time} {footer}")



from datetime import datetime
from dotenv import load_dotenv
import os 

from rich import print

from googleapiclient.discovery import build
from google.oauth2 import service_account
from googleapiclient.errors import HttpError

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email import header

import smtplib
import json
import requests
import pandas as pd

# from pydrive2.auth import GoogleAuth
# from pydrive2.drive import GoogleDrive
# from pydrive2.files import FileNotUploadedError

from requests_oauthlib import OAuth1

import re

#from PyPDF2 import PdfReader

from google.cloud import bigquery

import time

# from openai import OpenAI
# from openai import OpenAIError
    
from fajitas_tools import compute_log as log

ids_slack = {
    "Claire": {"id":os.getenv("slack_claire_id"), "col":"#FF9F1C"},
    "Alexis": {"id":os.getenv("slack_alexis_id"), "col":"#17C3B2"},
    "Brice":  {"id":os.getenv("slack_brice_id"), "col":"#083D77"},
}

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
      
def API_GOOGLE_SHEET_UPDATE_LINES(account_google_secret, google_sheet_id, df, sheet_name="Sheet1", range = "A1:ZZ"):
    """
        Fonction permettant de mettre à jour un onglet Google Sheet avec un dataframe
        Args
            account_google_secret: str, chemin vers le fichier secret de l'API Google
            google_sheet_id: str, identifiant de la feuille Google
            sheet_name: str, nom de l'onglet à mettre à jour. default: "Sheet1"
            range: str, plage de cellules à mettre à jour. default: "A1:ZZ"
            df: DataFrame, dataframe à mettre à jour

        Returns
            None
    """
    

    SCOPES = ['https://www.googleapis.com/auth/spreadsheets']
    SERVICE_ACCOUNT_FILE = account_google_secret
    creds = None
    creds = service_account.Credentials.from_service_account_file(
			SERVICE_ACCOUNT_FILE, scopes=SCOPES)
    
    spreadsheet_id = google_sheet_id
    sheet_name = sheet_name
    
    service = build('sheets', 'v4', credentials=creds)

    clear_request = service.spreadsheets().values().clear(
    spreadsheetId=spreadsheet_id,
    range=f'{sheet_name}!{range}',
)
    clear_response = clear_request.execute()

    # Conversion du dataframe en liste de listes
    header = list(df.columns)
    data = df.values.tolist()
    data.insert(0, header)

    #range_name = f'{sheet_name}!A1:{chr(ord("A") + df.shape[1] - 1)}{df.shape[0] + 1}'
    range_name = f'{sheet_name}!{range}'

    try:
        request = service.spreadsheets().values().update(
            spreadsheetId=spreadsheet_id,
            range=range_name,
            valueInputOption='RAW',
            body={'values': data}
        )

        response = request.execute()

        header = "API_GOOGLE_SHEET_UPDATE_LINES"
        msg = f"""
            Id Gsheet:\t\t{response['spreadsheetId']}
            Range:\t\t{response['updatedRange']}
            Row updated:\t\t{response['updatedRows']}
            Column updated:\t{response['updatedColumns']}
            Cells updated:\t{response['updatedCells']}
                """
        footer = "Fin de l'execution de la fonction"
        log.computelog(header, msg, footer)
    except HttpError as error:
        print(f"An error occurred: {error}")

def API_GOOGLE_SHEET_GET_FILE (account_google_secret, google_sheet_id, range):
    """
    Fonction permettant de récupérer les données d'un onglet Google Sheet
    Args
        account_google_secret: str, chemin vers le fichier secret de l'API Google
        google_sheet_id: str, identifiant de la feuille Google
        range: str, plage de cellules à récupérer
    Returns
        values: list, liste des valeurs récupérées
    """
    SCOPES = ['https://www.googleapis.com/auth/spreadsheets']
    SERVICE_ACCOUNT_FILE = account_google_secret
    creds = None
    creds = service_account.Credentials.from_service_account_file(
			SERVICE_ACCOUNT_FILE, scopes=SCOPES)
    SAMPLE_SPREADSHEET_ID = google_sheet_id
    SAMPLE_RANGE_NAME = range
    try:
        service = build('sheets', 'v4', credentials=creds)
		# Call the Sheets API
        sheet = service.spreadsheets()
        result = sheet.values().get(spreadsheetId=SAMPLE_SPREADSHEET_ID,
								range=SAMPLE_RANGE_NAME).execute()
        values = result.get('values', [])
        print(f"{datetime.now()}\tExecution OK !")
        header_log = "API_GOOGLE_SHEET_GET_FILE"
        msg_log = f"{datetime.now()}\tExecution OK !\n"
        footer_log = "Fin de l'execution de la fonction"
        log.computelog(header_log, msg_log, footer_log)
        return values
    except HttpError as error:
        print(f"An error occurred: {error}")
    